#!/usr/bin/env python

##*****************************************************************************
##
## File: createClip.py
##
## Copyright (c) 2016 Autodesk, Inc.
## All rights reserved.
##
## Use of this software is subject to the terms of the Autodesk license
## agreement provided at the time of installation or download, or which
## otherwise accompanies this software in either electronic or hard copy form.
##
##*****************************************************************************

"""
Wiretap SDK sample python program to create a clip.
"""

from __future__ import print_function
from __future__ import absolute_import

import sys
import getopt

from adsk.libwiretapPythonClientAPI import WireTapClient
from adsk.libwiretapPythonClientAPI import WireTapServerHandle
from adsk.libwiretapPythonClientAPI import WireTapNodeHandle
from adsk.libwiretapPythonClientAPI import WireTapClipFormat


class WireTapException(Exception):
    pass


def main(argv):

    # check the command line options for correctness
    try:
        opts, _ = getopt.getopt(argv, "h:n:")
    except getopt.GetoptError:
        print("Usage:\n" "-h <hostname>\n" "-n <node Id>")
        sys.exit(2)

    # default host name and node ID
    host_name = "localhost"
    node_id = ""

    # parse command line option to set specified host name
    # and node ID
    for opt, arg in opts:
        if opt == "-h":
            host_name = arg
        elif opt == "-n":
            node_id = arg

    # Initialize the Wiretap Client API.
    #
    wiretap_client = WireTapClient()
    if not wiretap_client.init():
        raise WireTapException("Unable to initialize Wiretap client API.")

    create_clip(host_name, node_id)


def create_clip(host_name, node_id):

    # Instantiate a server handle
    #
    server_handle = WireTapServerHandle(host_name)

    # Instantiate the clip node handle for the given server and node ID.
    #
    parent_node_handle = WireTapNodeHandle(server_handle, node_id)

    # Define a clip format. The parameters allow the server to
    # determine the frame buffer size for the specified frame format
    # (in this example, an NTSC raw RGB frame). The buffer size calculated
    # by the server can be obtained after the clip has been created.
    #
    clip_format = WireTapClipFormat(
        720,
        486,  # width, height
        3 * 8,  # bits per pixel
        3,  # number of channels
        29.97,  # frame rate
        0.9,  # pixel ratio
        WireTapClipFormat.ScanFormat.SCAN_FORMAT_FIELD_1_ODD,
        WireTapClipFormat.FORMAT_RGB(),
    )

    # Create a new node of type "CLIP" (a node type defined by
    # the IFFFS Wiretap Server).  Each Wiretap server defines a set of
    # node types (specified as string constants).
    #
    clip_node_handle = WireTapNodeHandle()
    if not parent_node_handle.createClipNode(
        "MyNewClip",  # display name
        clip_format,  # clip format
        "CLIP",  # extended (server-specific) type
        clip_node_handle,  # created node returned here
    ):
        raise WireTapException(
            "Unable to create clip node: %s." % parent_node_handle.lastError()
        )

    # Set the number of frames.
    #
    num_frames = 5
    if not clip_node_handle.setNumFrames(num_frames):
        raise WireTapException(
            "Unable to set the number of frames: %s." % clip_node_handle.lastError()
        )

    # Get the clip format back from the clip node. Even though we defined the
    # format above, we must query the updated format to get the size of the
    # frame buffer calculated by the server.  The server will also fill in
    # any format metadata that was not specified (for example: timecode,
    # drop mode, and so on).
    #
    if not clip_node_handle.getClipFormat(format):
        raise WireTapException(
            "Unable to obtain clip format: %s." % clip_node_handle.lastError()
        )

    # buf is a buffer initialized to zero
    #
    buf = "0" * clip_format.frameBufferSize()

    # Write the frames.
    #
    for frame_number in range(0, num_frames):
        if not clip_node_handle.writeFrame(
            frame_number, buf, clip_format.frameBufferSize()
        ):
            raise WireTapException(
                "Unable to obtain write frame %i: %s."
                % (frame_number, clip_node_handle.lastError())
            )
        # Print the frame info.
        #
        print("Successfully wrote frame %i." % frame_number)


if __name__ == "__main__":
    main(sys.argv[1:])
