#!/usr/bin/env python

##*****************************************************************************
##
## File: duplicateNode.py
##
## Copyright (c) 2016 Autodesk, Inc.
## All rights reserved.
##
## Use of this software is subject to the terms of the Autodesk license
## agreement provided at the time of installation or download, or which
## otherwise accompanies this software in either electronic or hard copy form.
##
##*****************************************************************************

"""
Wiretap SDK sample python program to duplicate a node.
"""

from __future__ import print_function
from __future__ import absolute_import

import sys
import getopt

from adsk.libwiretapPythonClientAPI import WireTapClient
from adsk.libwiretapPythonClientAPI import WireTapServerHandle
from adsk.libwiretapPythonClientAPI import WireTapServerId
from adsk.libwiretapPythonClientAPI import WireTapNodeHandle
from adsk.libwiretapPythonClientAPI import WireTapStr


class WireTapException(Exception):
    pass


def main(argv):

    # check the command line options for correctness
    try:
        opts, _ = getopt.getopt(argv, "h:n:p:d:")
    except getopt.GetoptError:
        print(
            "Usage:\n"
            "-h <hostname>\n"
            "-n <source node Id>"
            "-p <destination node Id>"
            "-d <display name>"
        )
        sys.exit(2)

    # default host name
    host_name = "localhost"
    node_id = ""
    parent_node_id = ""
    display_name = ""

    # parse command line option to set specified host name
    # and node ID
    for opt, arg in opts:
        if opt == "-h":
            host_name = arg
        elif opt == "-n":
            node_id = arg
        elif opt == "-p":
            parent_node_id = arg
        elif opt == "-d":
            display_name = arg

    # Initialize the Wiretap Client API.
    #
    wiretap_client = WireTapClient()
    if not wiretap_client.init():
        raise WireTapException("Unable to initialize Wiretap client API.")

    duplicate_node(host_name, node_id, parent_node_id, display_name)


def duplicate_node(host_name, node_id, parent_node_id, display_name):

    # Instantiate an IFFFS server handle.
    #
    ifffs_server_id = WireTapServerId("IFFFS", host_name)
    ifffs_server_handle = WireTapServerHandle(ifffs_server_id)

    # Instantiate the source node handle for the given server and node ID.
    #
    source_node_handle = WireTapNodeHandle(ifffs_server_handle, node_id)

    if len(display_name) == 0:
        source_display_name = WireTapStr()
        if not source_node_handle.getDisplayName(source_display_name):
            raise WireTapException(
                "Unable to obtain source node display name: %s"
                % source_node_handle.lastError()
            )
        display_name = "Duplicate of " + source_display_name.c_str()

    # Instantiate the destination node handle for the given server and node ID.
    #
    parent_node = WireTapNodeHandle(ifffs_server_handle, parent_node_id)

    new_node = WireTapNodeHandle()
    if not parent_node.duplicateNode(source_node_handle, display_name, new_node):
        raise WireTapException("Unable to duplicate node: %s" % parent_node.lastError())

    print(
        "Duplicated '%s' to '%s'"
        % (source_node_handle.getNodeId().id(), new_node.getNodeId().id())
    )


if __name__ == "__main__":
    main(sys.argv[1:])
