#!/usr/bin/env python

##*****************************************************************************
##
## File: createOpenClips.py
##
## Copyright (c) 2016 Autodesk, Inc.
## All rights reserved.
##
## Use of this software is subject to the terms of the Autodesk license
## agreement provided at the time of installation or download, or which
## otherwise accompanies this software in either electronic or hard copy form.
##
##*****************************************************************************

"""
Wiretap SDK sample python program to import all clip node in a directory to a
specified node on a server.
"""

from __future__ import print_function
from __future__ import absolute_import

import sys
import getopt

from adsk.libwiretapPythonClientAPI import WireTapClient
from adsk.libwiretapPythonClientAPI import WireTapServerHandle
from adsk.libwiretapPythonClientAPI import WireTapServerId
from adsk.libwiretapPythonClientAPI import WireTapNodeHandle
from adsk.libwiretapPythonClientAPI import WireTapClipFormat
from adsk.libwiretapPythonClientAPI import WireTapStr
from adsk.libwiretapPythonClientAPI import WireTapInt


class WireTapException(Exception):
    pass


def main(argv):
    # check the command line options for correctness
    try:
        opts, _ = getopt.getopt(argv, "h:n:d:")
    except getopt.GetoptError:
        print("Usage:\n" "-h <hostname>\n" "-n <node Id>" "-d <directory>")
        sys.exit(2)

    # default host name
    host_name = "localhost"
    parent_node_id = ""
    directory = ""

    # parse command line option to set specified host name
    # and node ID
    for opt, arg in opts:
        if opt == "-h":
            host_name = arg
        elif opt == "-n":
            parent_node_id = arg
        elif opt == "-d":
            directory = arg

    # Initialize the Wiretap Client API.
    #
    wiretap_client = WireTapClient()
    if not wiretap_client.init():
        raise WireTapException("Unable to initialize Wiretap client API.")

    import_open_clips(host_name, parent_node_id, directory)


def import_open_clips(host_name, parent_node_id, directory):

    # Instantiate a gateway server handle.
    #
    gateway_server_id = WireTapServerId("Gateway", host_name)
    gateway_server_handle = WireTapServerHandle(gateway_server_id)

    # Instantiate a gateway directory node handle.
    #
    directory_node_handle = WireTapNodeHandle(gateway_server_handle, directory)

    # Instantiate an IFFFS server handle.
    #
    ifffs_server_id = WireTapServerId("IFFFS", host_name)
    ifffs_server_handle = WireTapServerHandle(ifffs_server_id)

    # Instantiate a clip node handle for the given server and node ID.
    #
    parent_node_handle = WireTapNodeHandle(ifffs_server_handle, parent_node_id)

    # Loop thru all directory node children and import all clip nodes.
    #
    num_children = WireTapInt(0)
    if not directory_node_handle.getNumChildren(num_children):
        raise WireTapException(
            "Unable to obtain number of children: %s"
            % directory_node_handle.lastError()
        )

    for child_index in range(0, num_children):
        # Get the child node.
        #
        child_node_handle = WireTapNodeHandle()
        if not directory_node_handle.getChild(child_index, child_node_handle):
            raise WireTapException(
                "Unable to get child %d: %s"
                % (child_index, directory_node_handle.lastError())
            )

        # Get the child node type
        #
        child_type = WireTapStr()
        if not child_node_handle.getNodeTypeStr(child_type):
            raise WireTapException(
                "Unable to obtain node type: %s" % child_node_handle.lastError()
            )

        # Ignore non clip children
        #
        if child_type.c_str() == "CLIP":
            clip_name = WireTapStr()
        if not child_node_handle.getDisplayName(clip_name):
            raise WireTapException(
                "Unable to obtain node display name: %s" % child_node_handle.lastError()
            )

        source_data = WireTapStr()
        if not child_node_handle.getMetaData(
            "SourceData", "", 0, source_data  # stream name  # filter  # depth
        ):  # metadata
            raise WireTapException(
                "Unable to obtain node source data: %s" % child_node_handle.lastError()
            )

        # Define a clip format. The parameter allow the server to determine the
        # source information to import.
        #
        clip_format = WireTapClipFormat()
        clip_format.setMetaDataTag("SourceData")
        clip_format.setMetaData(source_data.c_str())

        # Create a new node of type "CLIP" (a node type defined by
        # the IFFFS Wiretap Server).  Each Wiretap server defines a set of
        # node types (specified as string constants).
        #
        clip_node_handle = WireTapNodeHandle()
        if not parent_node_handle.createClipNode(
            clip_name.c_str(),  # display name
            clip_format,  # clip format
            "CLIP",  # node type (server-specific)
            clip_node_handle,  # created node returned here
        ):
            raise WireTapException(
                "Unable to create clip node: %s." % parent_node_handle.lastError()
            )

        print("Imported '%s' to '%s'" % (clip_name, clip_node_handle.getNodeId().id()))


if __name__ == "__main__":
    main(sys.argv[1:])
