#!/usr/bin/env python

##*****************************************************************************
##
## File: listChildren.py
##
## Copyright (c) 2016 Autodesk, Inc.
## All rights reserved.
##
## Use of this software is subject to the terms of the Autodesk license
## agreement provided at the time of installation or download, or which
## otherwise accompanies this software in either electronic or hard copy form.
##
##*****************************************************************************

"""
Wiretap SDK sample python program to list all child nodes of a specified node
on a server.
"""

from __future__ import print_function
from __future__ import absolute_import

import sys
import getopt

from adsk.libwiretapPythonClientAPI import WireTapClient
from adsk.libwiretapPythonClientAPI import WireTapServerHandle
from adsk.libwiretapPythonClientAPI import WireTapNodeHandle
from adsk.libwiretapPythonClientAPI import WireTapStr
from adsk.libwiretapPythonClientAPI import WireTapInt


class WireTapException(Exception):
    pass


def main(argv):

    # check the command line options for correctness
    try:
        opts, _ = getopt.getopt(argv, "h:n:")
    except getopt.GetoptError:
        print("Usage:\n" "-h <hostname>\n" "-n <node Id>")
        sys.exit(2)

    # default host name
    host_name = "localhost"
    node_id = ""

    # parse command line option to set specified host name
    # and node ID
    for opt, arg in opts:
        if opt == "-h":
            host_name = arg
        elif opt == "-n":
            node_id = arg

    # Initialize the Wiretap Client API.
    #
    wiretap_client = WireTapClient()
    if not wiretap_client.init():
        raise WireTapException("Unable to initialize Wiretap client API.")

    list_children(host_name, node_id)


def list_children(host_name, node_id):

    try:
        # Instantiate a server handle
        #
        server_handle = WireTapServerHandle(host_name)

        # Obtain the root node of the server.
        #
        parent_node_handle = WireTapNodeHandle(server_handle, node_id)

        # Iterate across the first level of children.
        #
        num_children = WireTapInt(0)
        if not parent_node_handle.getNumChildren(num_children):
            raise WireTapException(
                "Unable to obtain number of children: %s"
                % parent_node_handle.lastError()
            )

        child = WireTapNodeHandle()
        child_name = WireTapStr()
        child_type_str = WireTapStr()
        for child_index in range(0, num_children):
            # Get the child node.
            #
            parent_node_handle.getChild(child_index, child)

            # Get the node's display name and type.
            #
            if not child.getDisplayName(child_name):
                raise WireTapException(
                    "Unable to obtain node name: %s." % child.lastError()
                )

            if not child.getNodeTypeStr(child_type_str):
                raise WireTapException(
                    "Unable to obtain node type: %s." % child.lastError()
                )

            # Print the node info.
            #
            print("Node: '%s' type: %s" % (child_name.c_str(), child_type_str.c_str()))

    finally:
        server_handle = None
        parent_node_handle = None


if __name__ == "__main__":
    main(sys.argv[1:])
