#!/usr/bin/env python

##*****************************************************************************
##
## File: listFrames.py
##
## Copyright (c) 2016 Autodesk, Inc.
## All rights reserved.
##
## Use of this software is subject to the terms of the Autodesk license
## agreement provided at the time of installation or download, or which
## otherwise accompanies this software in either electronic or hard copy form.
##
##*****************************************************************************

"""
Wiretap SDK sample python program to list all frames IDs and frame paths in
a clip.
"""

from __future__ import print_function
from __future__ import absolute_import

import sys
import getopt

from adsk.libwiretapPythonClientAPI import WireTapClient
from adsk.libwiretapPythonClientAPI import WireTapServerHandle
from adsk.libwiretapPythonClientAPI import WireTapNodeHandle
from adsk.libwiretapPythonClientAPI import WireTapStr
from adsk.libwiretapPythonClientAPI import WireTapInt


class WireTapException(Exception):
    pass


def main(argv):
    # check the command line options for correctness
    try:
        opts, _ = getopt.getopt(argv, "h:n:")
    except getopt.GetoptError:
        print("Usage:\n" "-h <hostname>\n" "-n <node Id>")
        sys.exit(2)

    # default host name and node ID
    host_name = "localhost"
    node_id = ""

    # parse command line option to set specified host name
    # and node ID
    for opt, arg in opts:
        if opt == "-h":
            host_name = arg
        elif opt == "-n":
            node_id = arg

    # Initialize the Wiretap Client API.
    #
    wiretap_client = WireTapClient()
    if not wiretap_client.init():
        raise WireTapException("Unable to initialize Wiretap client API.")

    list_frames(host_name, node_id)


def list_frames(host_name, node_id):

    # Instantiate a server handle
    #
    server_handle = WireTapServerHandle(host_name)

    # Instantiate the clip node handle for the given server and node ID.
    #
    clip_node_handle = WireTapNodeHandle(server_handle, node_id)

    # Get the list of clip frames.
    #
    num_frames = WireTapInt(0)
    if not clip_node_handle.getNumFrames(num_frames):
        raise WireTapException(
            "Unable to obtain number of frames: %s." % clip_node_handle.lastError()
        )

    # Read all frames ID and paths.
    #
    for frame_index in range(0, num_frames):
        # A frame ID is an opaque string, unique to the server.
        #
        frame_id = WireTapStr()
        if not clip_node_handle.getFrameId(frame_index, frame_id):
            raise WireTapException(
                "Unable to obtain frame ID %d: %s."
                % (frame_id, clip_node_handle.lastError())
            )

        # Frame IDs have an optional path representation. If paths are
        # available, a Wiretap client can use them to access frames directly,
        # instead of calling readFrame(). The frame path will be an empty string
        # if a path representation of the frame ID does not exist.
        # If path translation services are enabled and configured,
        # the mount point of the current machine will be used.
        #
        frame_id_path = WireTapStr()
        if not clip_node_handle.getFrameIdPath(frame_id, frame_id_path):
            raise WireTapException(
                "Unable to obtain frame ID path %d: %s."
                % (frame_id, clip_node_handle.lastError())
            )

        print(
            "Frame %d. ID: '%s'  File Path: %s."
            % (
                frame_index,
                frame_id.c_str(),
                frame_id_path.c_str() if frame_id_path.length() == 0 else "none",
            )
        )


if __name__ == "__main__":
    main(sys.argv[1:])
