#!/usr/bin/env python

# Wiretap SDK tool for listing all child nodes of a specified parent node.

from __future__ import print_function

import getopt
import sys
from libwiretapPythonClientAPI import (
    WireTapClient,
    WireTapServerHandle,
    WireTapNodeHandle,
    WireTapInt,
    WireTapStr,
)


def printUsage():
    print(
        "Usage:\n"
        "-n <node Id>\n"
        "[-h <hostname> (default = localhost)]\n"
        "[-N ] show display names (default = show node IDs)\n"
        "[-d <delimiter character> (default = none)]\n"
        "[-s <separator character> (default = newline)]\n"
    )


def main():

    # Check the command line options for correctness.
    try:
        opts, _ = getopt.getopt(sys.argv, "h:n:d:s:N")
    except getopt.GetoptError:
        printUsage()
        sys.exit(2)

    # Declare variables (to be set from command line arguments).
    # For hostName, the default "localhost" will work if a Wiretap server
    # is running on your machine.
    #
    nodeId = ""
    hostName = "localhost"
    delimiter = r"\0"
    separator = r"\n"
    showDisplayNames = 0

    # Parse command line and set variables just declared.
    for opt, arg in opts:
        if opt == "-h":
            hostName = arg
        elif opt == "-n":
            nodeId = arg
        elif opt == "-N":
            showDisplayNames = 1
        elif opt == "-d":
            delimiter = arg[0]
        elif opt == "-s":
            separator = arg[0]

    if nodeId == "":
        print("Parent node ID not specified.\n")
        printUsage()
        sys.exit(2)

    # Initialize the Wiretap Client API.
    #
    wiretapClient = WireTapClient()
    if not wiretapClient.init():
        raise Exception("Unable to initialize Wiretap client API")

    try:
        # Instantiate a server handle
        #
        server = WireTapServerHandle(hostName)

        # Instantiate the parent node
        #
        parent = WireTapNodeHandle(server, nodeId)
        numChildren = WireTapInt(0)

        if not parent.getNumChildren(numChildren):
            raise Exception(
                "Unable to obtain number of children: %s." % parent.lastError()
            )

        # Iterate across the first level of children.
        #
        i = 0
        child = WireTapNodeHandle()
        while i < numChildren:

            # Get the child node.
            #
            if not parent.getChild(i, child):
                raise Exception("Unable to retreive child: %s " % parent.lastError())

            # Get the node's ID or name
            #
            name = WireTapStr(child.getNodeId().id())
            if showDisplayNames:
                if not child.getDisplayName(name):
                    raise Exception(
                        "Unable to obtain node name: %s." % child.lastError()
                    )

            # Print the node info.
            #
            if i == int(numChildren) - 1:
                sep = "\n"
            else:
                sep = separator

            if delimiter != r"\0":
                print("%c%s%c%c" % (delimiter, name.c_str(), delimiter, sep), endl="")
            else:
                print("%s%c" % (name.c_str(), sep), endl="")

            i = i + 1

    finally:
        # Must destroy WireTapServerHandle and WireTapNodeHandle before
        # uninitializing the Wiretap Client API.
        #
        child = None
        parent = None
        server = None


if __name__ == "__main__":
    main()
