#!/usr/bin/env python

#
# Wiretap SDK tool for obtaining the metadata associated with a node.
#

from __future__ import print_function

import getopt
import sys
from libwiretapPythonClientAPI import (
    WireTapClient,
    WireTapServerHandle,
    WireTapNodeHandle,
    WireTapMetaData,
    WireTapStr,
)


def printUsage():
    print(
        "Usage:\n"
        "-n <node Id>\n"
        "[-h <hostname> (default = localhost)]\n"
        "[-s <metadata stream name (default = 'XML')>]\n"
        "[-f <metadata filter>]\n"
        "[-d <depth> (default = 1)]\n"
    )


def main():

    # Check the command line options for correctness.
    try:
        opts, _ = getopt.getopt(sys.argv, "h:n:d:s:f:")
    except getopt.GetoptError:
        printUsage()
        sys.exit(2)

    # Declare variables (to be set from command line arguments).
    # For hostName, the defaultlocalhost" will work if a Wiretap server
    # is running on your machine.
    #
    nodeId = ""
    hostName = "localhost"
    streamName = WireTapMetaData.STREAM_XML()
    filt = ""
    depth = 1

    # Parse command line and set variables just declared.
    #
    for opt, arg in opts:
        if opt == "-h":
            hostName = arg
        elif opt == "-n":
            nodeId = arg
        elif opt == "-d":
            depth = int(arg)
        elif opt == "-s":
            streamName = arg
        elif opt == "-f":
            filt = arg

    if nodeId == "":
        print("Node ID not specified.\n")
        printUsage()
        sys.exit(2)

    # Initialize the Wiretap Client API.
    #
    wiretapClient = WireTapClient()
    if not wiretapClient.init():
        raise Exception("Unable to initialize Wiretap client API")

    try:
        # Instantiate a server handle
        #
        server = WireTapServerHandle(hostName)

        # Instantiate the node handle and get its metadata
        #
        node = WireTapNodeHandle(server, nodeId)
        metaData = WireTapStr()
        if not node.getMetaData(streamName, filt, depth, metaData):
            raise Exception("Unable to retrieve meta data: %s" % node.lastError())

        print(metaData.c_str())

    finally:
        # Must destroy WireTapServerHandle and WireTapNodeHandle before
        # uninitializing the Wiretap Client API.
        #
        node = None
        server = None


if __name__ == "__main__":
    main()
